@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2011, Yubico AB
+ * Copyright (c) 2011-2012 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,12 +42,14 @@
 #include <ykcore.h>
 #include <ykcore_lcl.h>
 */
+#include <ykpers-args.h>
 
 /* duplicated from ykpers.c */
 struct ykp_config_t {
 	unsigned int yk_major_version;
 	unsigned int yk_minor_version;
-	unsigned int configuration_number;
+	unsigned int yk_build_version;
+	unsigned int command;
 
 	struct config_st *ykcore_config;
 };
@@ -111,6 +113,8 @@ int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, char **argv)
 
 	/* Options */
 	char *salt = NULL;
+	char ndef[128];
+	char ndef_type = NULL;
 
 	int rc;
 
@@ -120,13 +124,13 @@ int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, char **argv)
 	/* copy version number from st into cfg */
 	assert(ykp_configure_for(cfg, 1, st) == 1);
 
-	/* call args_to_config from ykpersonalize.c with a fake set of program arguments */
-	rc = args_to_config(argc, argv, cfg,
+	/* call args_to_config from ykpers-args.c with a fake set of program arguments */
+	rc = args_to_config(argc, argv, cfg, yk,
 			    &infname, &outfname,
 			    &autocommit, salt,
 			    st, &verbose,
 			    access_code, &use_access_code,
-			    &aesviahash,
+			    &aesviahash, &ndef_type, ndef,
 			    &exit_code);
 
 	return rc;
@@ -153,7 +157,7 @@ YK_STATUS * _test_init_st(int major, int minor, int build)
  */
 int _parse_args_rc(int argc, char *argv[])
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
 	int rc = 0;
 
@@ -167,7 +171,7 @@ int _parse_args_rc(int argc, char *argv[])
 
 int _test_config_slot1(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(1, 3, 0);
 	int rc = 0;
 	struct config_st *ycfg;
@@ -198,7 +202,7 @@ int _test_config_slot1(void)
 
 int _test_config_static_slot2(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 0, 0);
 	int rc = 0;
 	struct config_st *ycfg;
@@ -229,7 +233,7 @@ int _test_config_static_slot2(void)
 
 int _test_too_old_key(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(1, 3, 0);
 	int rc = 0;
 
@@ -249,7 +253,7 @@ int _test_too_old_key(void)
 
 int _test_too_new_key(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
 	int rc = 0;
 
@@ -269,7 +273,7 @@ int _test_too_new_key(void)
 
 int _test_non_config_args(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
 	int rc = 0;
 
@@ -286,26 +290,29 @@ int _test_non_config_args(void)
 
 	/* Options */
 	char *salt = NULL;
+	char ndef[128];
+	char ndef_type = NULL;
 
 	char *argv[] = {
-		"unittest", "-sout", "-iin", "-c313233343536", "-y", "-v",
+		"unittest", "-1", "-sout", "-iin", "-c313233343536", "-y", "-v",
 		NULL
 	};
-	int argc = 6;
+	int argc = 7;
 
 	ykp_errno = 0;
 	optind = 0; /* getopt reinit */
 
 	/* copy version number from st into cfg */
-	assert(ykp_configure_for(cfg, 1, st) == 1);
+  ykp_configure_version(cfg, st);
+	//assert(ykp_configure_for(cfg, 1, st) == 1);
 
-	/* call args_to_config from ykpersonalize.c with a fake set of program arguments */
-	rc = args_to_config(argc, argv, cfg,
+	/* call args_to_config from ykpers-args.c with a fake set of program arguments */
+	rc = args_to_config(argc, argv, cfg, yk,
 			    &infname, &outfname,
 			    &autocommit, salt,
 			    st, &verbose,
 			    access_code, &use_access_code,
-			    &aesviahash,
+			    &aesviahash, &ndef_type, ndef,
 			    &exit_code);
 	assert(rc == 1);
 	i = strcmp(infname, "in"); assert(i == 0);
@@ -320,7 +327,7 @@ int _test_non_config_args(void)
 
 int _test_oath_hotp_nist_160_bits(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 1, 0);
 	int rc = 0;
 	struct config_st *ycfg;
@@ -351,7 +358,7 @@ int _test_oath_hotp_nist_160_bits(void)
 
 int _test_extended_flags1(void)
 {
-	YKP_CONFIG *cfg = ykp_create_config();
+	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
 	int rc = 0;
 
@@ -473,6 +480,93 @@ int _test_uid_for_chal_resp(void)
 	assert(rc == 0);
 }
 
+int _test_swap_with_slot(void)
+{
+	/* Test that you can not both swap and set slot */
+	char *argv[] = {
+		"unittest", "-x", "-1",
+		NULL
+	};
+	int rc = _parse_args_rc (3, argv);
+	assert(rc == 0);
+}
+
+int _test_slot_with_update(void)
+{
+	/* Test the update must be before slot */
+	char *argv[] = {
+		"unittest", "-1", "-u",
+		NULL
+	};
+	int rc = _parse_args_rc (3, argv);
+	assert(rc == 0);
+}
+
+int _test_swap_with_update(void)
+{
+	/* Test the update must be before slot */
+	char *argv[] = {
+		"unittest", "-u", "-x",
+		NULL
+	};
+	int rc = _parse_args_rc (3, argv);
+	assert(rc == 0);
+}
+
+int _test_ndef_for_neo(void)
+{
+	YKP_CONFIG *cfg = ykp_alloc();
+	YK_STATUS *st = _test_init_st(2, 1, 7);
+
+	char *argv[] = {
+		"unittest", "-nhttps://my.yubico.com/neo/",
+		NULL
+	};
+	int argc = 2;
+
+	int rc = _test_config(cfg, st, argc, argv);
+	assert(rc == 1);
+	struct config_st *ycfg = (struct config_st *) ykp_core_config(cfg);
+	assert(((struct ykp_config_t*)cfg)->command == SLOT_NDEF);
+
+	ykp_free_config(cfg);
+	free(st);
+}
+
+int _test_ndef_with_non_neo(void)
+{
+	YKP_CONFIG *cfg = ykp_alloc();
+	YK_STATUS *st = _test_init_st(2, 2, 4);
+
+	char *argv[] = {
+		"unittest", "-nhttps://my.yubico.com/neo/",
+		NULL
+	};
+	int argc = 2;
+
+	int rc = _test_config(cfg, st, argc, argv);
+	assert(rc == 0);
+
+	ykp_free_config(cfg);
+	free(st);
+}
+
+int _test_slot_two_with_neo(void)
+{
+	YKP_CONFIG *cfg = ykp_alloc();
+	YK_STATUS *st = _test_init_st(2, 1, 7);
+
+	char *argv[] = {
+		"unittest", "-2", NULL
+	};
+	int argc = 2;
+
+	int rc = _test_config(cfg, st, argc, argv);
+	assert(rc == 0);
+	ykp_free_config(cfg);
+	free(st);
+}
+
 int main (int argc, char **argv)
 {
 	_test_config_slot1();
@@ -490,6 +584,12 @@ int main (int argc, char **argv)
 	_test_key_mixed_case1();
 	_test_uid_for_oath();
 	_test_uid_for_chal_resp();
+	_test_swap_with_slot();
+	_test_slot_with_update();
+	_test_swap_with_update();
+	_test_ndef_for_neo();
+	_test_ndef_with_non_neo();
+	_test_slot_two_with_neo();
 
 	return 0;
 }
