@@ -1,7 +1,7 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008, 2009, 2010, 2011, Yubico AB
- * Copyright (c) 2010  Tollef Fog Heen <tfheen@err.no>
+ * Copyright (c) 2008-2012 Yubico AB
+ * Copyright (c) 2010 Tollef Fog Heen <tfheen@err.no>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stddef.h>
@@ -36,12 +37,19 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include <ykcore_lcl.h>
 #include <ykpers.h>
 #include <yubikey.h> /* To get yubikey_modhex_encode and yubikey_hex_encode */
 #include <ykdef.h>
 
+#define YUBICO_OATH_VENDOR_ID_HEX	0xe1	/* UB as hex */
+#define YUBICO_HOTP_EVENT_TOKEN_TYPE	0x63	/* HE as hex */
+
 const char *usage =
 "Usage: ykpersonalize [options]\n"
+"-u        update configuration without overwriting.  This is only available\n"
+"          in YubiKey 2.3 and later.  EXTFLAG_ALLOW_UPDATE will be set by\n"
+"          default\n"
 "-1        change the first configuration.  This is the default and\n"
 "          is normally used for true OTP generation.\n"
 "          In this configuration, TKTFLAG_APPEND_CR is set by default.\n"
@@ -50,6 +58,8 @@ const char *usage =
 "          In this configuration, TKTFLAG_APPEND_CR, CFGFLAG_STATIC_TICKET,\n"
 "          CFGFLAG_STRONG_PW1, CFGFLAG_STRONG_PW2 and CFGFLAG_MAN_UPDATE\n"
 "          are set by default.\n"
+"-x        swap the configuration in slot 1 and 2.  This is for YubiKey 2.3\n"
+"          and newer only\n"
 "-sFILE    save configuration to FILE instead of key.\n"
 "          (if FILE is -, send to stdout)\n"
 "-iFILE    read configuration from FILE.\n"
@@ -58,6 +68,8 @@ const char *usage =
 "          char hex value (not modhex)\n"
 "-cXXX..   A 12 char hex value (not modhex) to use as access code for programming\n"
 "          (this does NOT SET the access code, that's done with -oaccess=)\n"
+"-nXXX..   Write NDEF type 2 URI to YubiKey NEO, must be used on it's own\n"
+"-tXXX..   Write NDEF type 2 text to YubiKey NEO, must be used on it's own\n"
 "-oOPTION  change configuration option.  Possible OPTION arguments are:\n"
 "          salt=ssssssss       Salt to be used when deriving key from a\n"
 "                              password.  If none is given, a unique random\n"
@@ -69,6 +81,7 @@ const char *usage =
 "          access=xxxxxxxxxxx  New access code to set, in HEX.\n"
 "                              MUST be 12 characters long.\n"
 "          oath-imf=IMF        OATH Initial Moving Factor to use.\n"
+"          oath-id[=h:OOTT...] OATH Token Identifier (none for serial-based)\n"
 "\n"
 "          Ticket flags for all firmware versions:\n"
 "          [-]tab-first           set/clear TAB_FIRST\n"
@@ -120,12 +133,58 @@ const char *usage =
 "          [-]serial-usb-visible  set/clear SERIAL_USB_VISIBLE\n"
 "          [-]serial-api-visible  set/clear SERIAL_API_VISIBLE\n"
 "\n"
+"          Extended flags for firmware version 2.3 and above:\n"
+"          [-]use-numeric-keypad  set/clear USE_NUMERIC_KEYPAD\n"
+"          [-]fast-trig           set/clear FAST_TRIG\n"
+"          [-]allow-update        set/clear ALLOW_UPDATE\n"
+"          [-]dormant             set/clear DORMANT\n"
+"\n"
 "-y        always commit (do not prompt)\n"
 "\n"
 "-v        verbose\n"
 "-h        help (this text)\n"
 ;
-const char *optstring = "12a:c:hi:o:s:vy";
+const char *optstring = "u12xa:c:n:t:hi:o:s:vy";
+
+static const YK_CONFIG default_config1 = {
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* fixed */
+        { 0, 0, 0, 0, 0, 0 },   /* uid */
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* key */
+        { 0, 0, 0, 0, 0, 0 },   /* accCode */
+        0,                      /* fixedSize */
+        0,                      /* extFlags */
+        TKTFLAG_APPEND_CR,      /* tktFlags */
+        0,                      /* cfgFlags */
+        0,                      /* ctrOffs */
+        0                       /* crc */
+};
+
+static const YK_CONFIG default_config2 = {
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* fixed */
+        { 0, 0, 0, 0, 0, 0 },   /* uid */
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* key */
+        { 0, 0, 0, 0, 0, 0 },   /* accCode */
+        0,                      /* fixedSize */
+        0,                      /* extFlags */
+        TKTFLAG_APPEND_CR,      /* tktFlags */
+        /* cfgFlags */
+        CFGFLAG_STATIC_TICKET | CFGFLAG_STRONG_PW1 | CFGFLAG_STRONG_PW2 | CFGFLAG_MAN_UPDATE,
+        0,                      /* ctrOffs */
+        0                       /* crc */
+};
+
+static const YK_CONFIG default_update = {
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* fixed */
+        { 0, 0, 0, 0, 0, 0 },   /* uid */
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* key */
+        { 0, 0, 0, 0, 0, 0 },   /* accCode */
+        0,                      /* fixedSize */
+        EXTFLAG_ALLOW_UPDATE,   /* extFlags */
+        TKTFLAG_APPEND_CR,      /* tktFlags */
+        0,                      /* cfgFlags */
+        0,                      /* ctrOffs */
+        0                       /* crc */
+};
 
 static int hex_modhex_decode(unsigned char *result, size_t *resultlen,
 			     const char *str, size_t strl,
@@ -189,12 +248,12 @@ void report_yk_error()
  *
  * Done in this way to be testable (see tests/test_args_to_config.c).
  */
-int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
+int args_to_config(int argc, char **argv, YKP_CONFIG *cfg, YK_KEY *yk,
 		   const char **infname, const char **outfname,
 		   bool *autocommit, char *salt,
 		   YK_STATUS *st, bool *verbose,
 		   unsigned char *access_code, bool *use_access_code,
-		   bool *aesviahash,
+		   bool *aesviahash, char *ndef_type, char *ndef,
 		   int *exit_code)
 {
 	int c;
@@ -203,6 +262,11 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 	bool slot_chosen = false;
 	bool mode_chosen = false;
 	bool option_seen = false;
+	bool swap_seen = false;
+	bool update_seen = false;
+	bool ndef_seen = false;
+
+	ykp_configure_version(cfg, st);
 
 	struct config_st *ycfg = (struct config_st *) ykp_core_config(cfg);
 
@@ -240,35 +304,91 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 		}
 
 		switch (c) {
-		case '1':
+		case 'u':
 			if (slot_chosen) {
-				fprintf(stderr, "You may only choose slot (-1 / -2) once.\n");
+				fprintf(stderr, "You must use update before slot (-1 / -2).\n");
 				*exit_code = 1;
 				return 0;
 			}
-			if (option_seen) {
-				fprintf(stderr, "You must choose slot before any options (-o).\n");
+			if (swap_seen) {
+				fprintf(stderr, "Update (-u) and swap (-x) can't be combined.\n");
 				*exit_code = 1;
 				return 0;
 			}
-			if (!ykp_configure_for(cfg, 1, st))
+			if (ndef_seen) {
+				fprintf(stderr, "Update (-u) can not be combined with ndef (-n).\n");
+				*exit_code = 1;
 				return 0;
-			slot_chosen = true;
+			}
+			update_seen = true;
 			break;
-		case '2':
+		case '1':
+		case '2': {
+				if (slot_chosen) {
+					fprintf(stderr, "You may only choose slot (-1 / -2) once.\n");
+					*exit_code = 1;
+					return 0;
+				}
+				if (option_seen) {
+					fprintf(stderr, "You must choose slot before any options (-o).\n");
+					*exit_code = 1;
+					return 0;
+				}
+				if (swap_seen) {
+					fprintf(stderr, "You can not combine slot swap (-x) with configuring a slot.\n");
+					*exit_code = 1;
+					return 0;
+				}
+				if (ndef_seen) {
+					fprintf(stderr, "Slot (-1 / -2) can not be combined with ndef (-n)\n");
+					*exit_code = 1;
+					return 0;
+				}
+				int command;
+				if (update_seen) {
+					memcpy(ycfg, &default_update, sizeof(default_update));
+					if(c == '1') {
+						command = SLOT_UPDATE1;
+					} else if(c == '2') {
+						command = SLOT_UPDATE2;
+					}
+				} else if (c == '1') {
+					command = SLOT_CONFIG;
+					memcpy(ycfg, &default_config1, sizeof(default_config1));
+				} else if (c == '2') {
+					command = SLOT_CONFIG2;
+					memcpy(ycfg, &default_config2, sizeof(default_config2));
+				}
+				if (!ykp_configure_command(cfg, command))
+					return 0;
+				slot_chosen = true;
+				break;
+			}
+		case 'x':
 			if (slot_chosen) {
-				fprintf(stderr, "You may only choose slot (-1 / -2) once.\n");
+				fprintf(stderr, "You can not use slot swap with a chosen slot (-1 / -2).\n");
 				*exit_code = 1;
 				return 0;
 			}
 			if (option_seen) {
-				fprintf(stderr, "You must choose slot before any options (-o).\n");
+				fprintf(stderr, "You must set slot swap before any options (-o).\n");
 				*exit_code = 1;
 				return 0;
 			}
-			if (!ykp_configure_for(cfg, 2, st))
+			if (update_seen) {
+				fprintf(stderr, "Update (-u) and swap (-x) can't be combined.\n");
+				*exit_code = 1;
 				return 0;
-			slot_chosen = true;
+			}
+			if (ndef_seen) {
+				fprintf(stderr, "Swap (-x) can not be combined with ndef (-n).\n");
+				*exit_code = 1;
+				return 0;
+			}
+			if (!ykp_configure_command(cfg, SLOT_SWAP)) {
+				return 0;
+			}
+			swap_seen = true;
 			break;
 		case 'i':
 			*infname = optarg;
@@ -299,25 +419,34 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 			*use_access_code = true;
 			break;
 		}
+		case 't':
+			*ndef_type = 'T';
+		case 'n':
+			if(!*ndef_type) {
+				*ndef_type = 'U';
+			}
+			if (slot_chosen || swap_seen || update_seen || option_seen) {
+				fprintf(stderr, "Ndef (-n/-t) must be used on it's own.\n");
+				*exit_code = 1;
+				return 0;
+			}
+			if (!ykp_configure_command(cfg, SLOT_NDEF)) {
+				return 0;
+			}
+			memcpy(ndef, optarg, strnlen(optarg, 128));
+			ndef_seen = true;
+			break;
 		case 'o':
 			if (strncmp(optarg, "salt=", 5) == 0)
 				salt = strdup(optarg+5);
 			else if (strncmp(optarg, "fixed=", 6) == 0) {
-				const char *fixed = optarg+6;
-				size_t fixedlen = strlen (fixed);
-				unsigned char fixedbin[256];
-				size_t fixedbinlen = 0;
-				int rc = hex_modhex_decode(fixedbin, &fixedbinlen,
-							   fixed, fixedlen,
-							   0, 16, true);
-				if (rc <= 0) {
+				if (_set_fixed(optarg + 6, cfg) != 1) {
 					fprintf(stderr,
 						"Invalid fixed string: %s\n",
-						fixed);
+						optarg + 6);
 					*exit_code = 1;
 					return 0;
 				}
-				ykp_set_fixed(cfg, fixedbin, fixedbinlen);
 			}
 			else if (strncmp(optarg, "uid=", 4) == 0) {
 				const char *uid = optarg+4;
@@ -442,6 +571,12 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 					return 0;
 				}
 			}
+			else if (strncmp(optarg, "oath-id=", 8) == 0 || strcmp(optarg, "oath-id") == 0) {
+				if (_set_oath_id(optarg, cfg, ycfg, yk, st) != 1) {
+					*exit_code = 1;
+					return 0;
+				}
+			}
 
 #define EXTFLAG(o, f)							\
 			else if (strcmp(optarg, o) == 0) {		\
@@ -458,6 +593,10 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 			EXTFLAG("serial-btn-visible", SERIAL_BTN_VISIBLE)
 			EXTFLAG("serial-usb-visible", SERIAL_USB_VISIBLE)
 			EXTFLAG("serial-api-visible", SERIAL_API_VISIBLE)
+      EXTFLAG("use-numeric-keypad", USE_NUMERIC_KEYPAD)
+      EXTFLAG("fast-trig", FAST_TRIG)
+			EXTFLAG("allow-update", ALLOW_UPDATE)
+      EXTFLAG("dormant", DORMANT)
 #undef EXTFLAG
 			else {
 				fprintf(stderr, "Unknown option '%s'\n",
@@ -481,6 +620,31 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 		}
 	}
 
+	if (!slot_chosen && !ndef_seen) {
+		fprintf(stderr, "A slot must be chosen with -1 or -2.\n");
+		*exit_code = 1;
+		return 0;
+	}
+
+	if (update_seen) {
+		struct config_st *core_config = (struct config_st *) ykp_core_config(cfg);
+		if ((core_config->tktFlags & TKTFLAG_UPDATE_MASK) != core_config->tktFlags) {
+			fprintf(stderr, "Unallowed ticket flags with update.\n");
+			*exit_code = 1;
+			return 0;
+		}
+		if ((core_config->cfgFlags & CFGFLAG_UPDATE_MASK) != core_config->cfgFlags) {
+			fprintf(stderr, "Unallowed cfg flags with update.\n");
+			*exit_code = 1;
+			return 0;
+		}
+		if ((core_config->extFlags & EXTFLAG_UPDATE_MASK) != core_config->extFlags) {
+			fprintf(stderr, "Unallowed ext flags with update.\n");
+			*exit_code = 1;
+			return 0;
+		}
+	}
+
 	if (*aesviahash) {
 		bool long_key_valid = false;
 		int res = 0;
@@ -510,3 +674,124 @@ int args_to_config(int argc, char **argv, YKP_CONFIG *cfg,
 
 	return 1;
 }
+
+int _set_fixed(char *optarg, YKP_CONFIG *cfg) {
+	const char *fixed = optarg;
+	size_t fixedlen = strlen (fixed);
+	unsigned char fixedbin[256];
+	size_t fixedbinlen = 0;
+	int rc = hex_modhex_decode(fixedbin, &fixedbinlen,
+				   fixed, fixedlen,
+				   0, 16, true);
+	if (rc <= 0)
+		return 0;
+
+	ykp_set_fixed(cfg, fixedbin, fixedbinlen);
+	return 1;
+}
+
+
+/* re-format decimal 12345678 into 'hex' 0x12 0x34 0x56 0x78 */
+int _format_decimal_as_hex(uint8_t *dst, size_t dst_len, uint8_t *src)
+{
+	uint8_t *end;
+
+	end = dst + dst_len;
+	while (src[0] && src[1]) {
+		if (dst >= end)
+			return 0;
+		*dst = ((src[0] - '0') * 0x10) + src[1] - '0';
+		dst++;
+		src += 2;
+	}
+
+	return 1;
+}
+
+/* For details, see YubiKey Manual 2010-09-16 section 5.3.4 - OATH-HOTP Token Identifier */
+int _format_oath_id(uint8_t *dst, size_t dst_len, uint8_t vendor, uint8_t type, uint32_t mui)
+{
+	uint8_t buf[8 + 1];
+
+	if (mui > 99999999)
+		return 0;
+
+	/* two bytes vendor and token type, and eight bytes MUI */
+	if (dst_len < 2 + 8)
+		return 0;
+
+	/* Make the YubiKey output the MUI number in decimal */
+	snprintf(buf, sizeof(buf), "%08i", mui);
+
+	dst[0] = vendor;
+	dst[1] = type;
+
+	if (_format_decimal_as_hex(dst + 2, dst_len - 2, buf) != 1)
+		return 0;
+
+	return 1;
+}
+
+int _set_oath_id(char *optarg, YKP_CONFIG *cfg, struct config_st *ycfg, YK_KEY *yk, YK_STATUS *st) {
+	/* For details, see YubiKey Manual 2010-09-16 section 5.3.4 - OATH-HOTP Token Identifier */
+	if (!(ycfg->tktFlags & TKTFLAG_OATH_HOTP) == TKTFLAG_OATH_HOTP) {
+		fprintf(stderr,
+			"Option oath-id= only valid with -ooath-hotp or -ooath-hotp8.\n"
+			);
+		return 0;
+	}
+	if (! ykp_set_cfgflag_OATH_FIXED_MODHEX2(cfg, true))
+		return 0;
+	if (! ykp_set_extflag_SERIAL_API_VISIBLE(cfg, true))
+		return 0;
+
+	if (strlen(optarg) > 7) {
+		if (_set_fixed(optarg + 8, cfg) != 1) {
+			fprintf(stderr,
+				"Invalid OATH token identifier %s supplied with oath-id=.\n", optarg + 8
+				);
+			return 0;
+		}
+	} else {
+		/* No Token Id supplied, try to create one automatically based on
+		 * the serial number of the YubiKey.
+		 */
+		unsigned int serial;
+		uint8_t oath_id[12] = {0};
+		if (ykds_version_major(st) > 2 ||
+		    (ykds_version_major(st) == 2 &&
+		     ykds_version_minor(st) >= 2)) {
+			if (! yk_get_serial(yk, 0, 0, &serial)) {
+				fprintf(stderr,
+					"YubiKey refuses reading serial number. "
+					"Can't use -ooath-id.\n"
+					);
+				return 0;
+			}
+		} else {
+			fprintf(stderr,
+				"YubiKey %d.%d.%d does not support reading serial number. "
+				"Can't use -ooath-id.\n",
+				ykds_version_major(st),
+				ykds_version_minor(st),
+				ykds_version_build(st)
+				);
+			return 0;
+		}
+
+		if (_format_oath_id(oath_id, sizeof(oath_id), YUBICO_OATH_VENDOR_ID_HEX,
+				    YUBICO_HOTP_EVENT_TOKEN_TYPE, serial) != 1) {
+			fprintf(stderr, "Failed formatting OATH token identifier.\n");
+			return 0;
+		}
+
+		if (ykp_set_fixed(cfg, oath_id, 6) != 1) {
+			fprintf(stderr,
+				"Failed setting OATH token identifier.\n"
+				);
+			return 0;
+		}
+	}
+
+	return 1;
+}
